/****************************************************************************
** Meta object code from reading C++ file 'qquick3dtextureproviderextension.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtquick3d/src/quick3d/extensions/qquick3dtextureproviderextension.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquick3dtextureproviderextension.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN32QQuick3DTextureProviderExtensionE_t {};
} // unnamed namespace

template <> constexpr inline auto QQuick3DTextureProviderExtension::qt_create_metaobjectdata<qt_meta_tag_ZN32QQuick3DTextureProviderExtensionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QQuick3DTextureProviderExtension",
        "QML.Element",
        "TextureProviderExtension",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "TextureProviderExtension is an abstract type",
        "QML.AddedInVersion",
        "1547",
        "samplerHintChanged",
        "",
        "surfaceChanged",
        "samplerHint",
        "SamplerHint",
        "Sampler2D",
        "Sampler2DArray",
        "Sampler3D",
        "SamplerCube",
        "SamplerCubeArray",
        "SamplerBuffer"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'samplerHintChanged'
        QtMocHelpers::SignalData<void()>(9, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'surfaceChanged'
        QtMocHelpers::RevisionedSignalData<void()>(11, 10, QMC::AccessPublic, 0x60c, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'samplerHint'
        QtMocHelpers::PropertyData<enum SamplerHint>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SamplerHint'
        QtMocHelpers::EnumData<enum SamplerHint>(13, 13, QMC::EnumIsScoped).add({
            {   14, SamplerHint::Sampler2D },
            {   15, SamplerHint::Sampler2DArray },
            {   16, SamplerHint::Sampler3D },
            {   17, SamplerHint::SamplerCube },
            {   18, SamplerHint::SamplerCubeArray },
            {   19, SamplerHint::SamplerBuffer },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<QQuick3DTextureProviderExtension, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QQuick3DTextureProviderExtension::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuick3DRenderExtension::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32QQuick3DTextureProviderExtensionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32QQuick3DTextureProviderExtensionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN32QQuick3DTextureProviderExtensionE_t>.metaTypes,
    nullptr
} };

void QQuick3DTextureProviderExtension::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QQuick3DTextureProviderExtension *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->samplerHintChanged(); break;
        case 1: _t->surfaceChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QQuick3DTextureProviderExtension::*)()>(_a, &QQuick3DTextureProviderExtension::samplerHintChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DTextureProviderExtension::*)()>(_a, &QQuick3DTextureProviderExtension::surfaceChanged, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum SamplerHint*>(_v) = _t->samplerHint(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSamplerHint(*reinterpret_cast<enum SamplerHint*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QQuick3DTextureProviderExtension::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuick3DTextureProviderExtension::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32QQuick3DTextureProviderExtensionE_t>.strings))
        return static_cast<void*>(this);
    return QQuick3DRenderExtension::qt_metacast(_clname);
}

int QQuick3DTextureProviderExtension::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuick3DRenderExtension::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void QQuick3DTextureProviderExtension::samplerHintChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuick3DTextureProviderExtension::surfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
