/****************************************************************************
** Meta object code from reading C++ file 'qquick3dskymaterial_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtquick3d/src/quick3d/qquick3dskymaterial_p.h"
#include <QtNetwork/QSslError>
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquick3dskymaterial_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19QQuick3DSkyMaterialE_t {};
} // unnamed namespace

template <> constexpr inline auto QQuick3DSkyMaterial::qt_create_metaobjectdata<qt_meta_tag_ZN19QQuick3DSkyMaterialE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QQuick3DSkyMaterial",
        "QML.Element",
        "SkyMaterial",
        "QML.AddedInVersion",
        "1548",
        "radianceMapSizeChanged",
        "",
        "fragmentShaderChanged",
        "fragmentShaderCodeChanged",
        "enableIBLChanged",
        "iblSampleCountChanged",
        "iblRenderFramesChanged",
        "skyboxModeChanged",
        "setRadianceMapSize",
        "radianceMapSize",
        "setFragmentShader",
        "QUrl",
        "newFragmentShader",
        "setFragmentShaderCode",
        "newFragmentShaderCode",
        "setEnableIBL",
        "newEnableIBL",
        "setIblSampleCount",
        "newIblSampleCount",
        "setIblRenderFrames",
        "newIblRenderFrames",
        "setSkyboxMode",
        "SkyboxMode",
        "skyboxMode",
        "fragmentShader",
        "fragmentShaderCode",
        "enableIBL",
        "iblSampleCount",
        "iblRenderFrames",
        "Cubemap",
        "ScreenSpaceFull",
        "ScreenSpaceHalf",
        "ScreenSpaceQuarter"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'radianceMapSizeChanged'
        QtMocHelpers::RevisionedSignalData<void()>(5, 6, QMC::AccessPublic, 0x60c, QMetaType::Void),
        // Signal 'fragmentShaderChanged'
        QtMocHelpers::RevisionedSignalData<void()>(7, 6, QMC::AccessPublic, 0x60c, QMetaType::Void),
        // Signal 'fragmentShaderCodeChanged'
        QtMocHelpers::RevisionedSignalData<void()>(8, 6, QMC::AccessPublic, 0x60c, QMetaType::Void),
        // Signal 'enableIBLChanged'
        QtMocHelpers::RevisionedSignalData<void()>(9, 6, QMC::AccessPublic, 0x60c, QMetaType::Void),
        // Signal 'iblSampleCountChanged'
        QtMocHelpers::RevisionedSignalData<void()>(10, 6, QMC::AccessPublic, 0x60c, QMetaType::Void),
        // Signal 'iblRenderFramesChanged'
        QtMocHelpers::RevisionedSignalData<void()>(11, 6, QMC::AccessPublic, 0x60c, QMetaType::Void),
        // Signal 'skyboxModeChanged'
        QtMocHelpers::RevisionedSignalData<void()>(12, 6, QMC::AccessPublic, 0x60c, QMetaType::Void),
        // Slot 'setRadianceMapSize'
        QtMocHelpers::RevisionedSlotData<void(int)>(13, 6, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'setFragmentShader'
        QtMocHelpers::RevisionedSlotData<void(const QUrl &)>(15, 6, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'setFragmentShaderCode'
        QtMocHelpers::RevisionedSlotData<void(const QString &)>(18, 6, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'setEnableIBL'
        QtMocHelpers::RevisionedSlotData<void(bool)>(20, 6, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Slot 'setIblSampleCount'
        QtMocHelpers::RevisionedSlotData<void(int)>(22, 6, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::Int, 23 },
        }}),
        // Slot 'setIblRenderFrames'
        QtMocHelpers::RevisionedSlotData<void(int)>(24, 6, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'setSkyboxMode'
        QtMocHelpers::RevisionedSlotData<void(enum SkyboxMode)>(26, 6, QMC::AccessPublic, 0x60c, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'radianceMapSize'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 0, 0x60c),
        // property 'fragmentShader'
        QtMocHelpers::PropertyData<QUrl>(29, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 1, 0x60c),
        // property 'fragmentShaderCode'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2, 0x60c),
        // property 'enableIBL'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 3, 0x60c),
        // property 'iblSampleCount'
        QtMocHelpers::PropertyData<int>(32, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4, 0x60c),
        // property 'iblRenderFrames'
        QtMocHelpers::PropertyData<int>(33, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5, 0x60c),
        // property 'skyboxMode'
        QtMocHelpers::PropertyData<enum SkyboxMode>(28, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 6, 0x60c),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SkyboxMode'
        QtMocHelpers::EnumData<enum SkyboxMode>(27, 27, QMC::EnumIsScoped).add({
            {   34, SkyboxMode::Cubemap },
            {   35, SkyboxMode::ScreenSpaceFull },
            {   36, SkyboxMode::ScreenSpaceHalf },
            {   37, SkyboxMode::ScreenSpaceQuarter },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QQuick3DSkyMaterial, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QQuick3DSkyMaterial::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuick3DObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QQuick3DSkyMaterialE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QQuick3DSkyMaterialE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19QQuick3DSkyMaterialE_t>.metaTypes,
    nullptr
} };

void QQuick3DSkyMaterial::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QQuick3DSkyMaterial *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->radianceMapSizeChanged(); break;
        case 1: _t->fragmentShaderChanged(); break;
        case 2: _t->fragmentShaderCodeChanged(); break;
        case 3: _t->enableIBLChanged(); break;
        case 4: _t->iblSampleCountChanged(); break;
        case 5: _t->iblRenderFramesChanged(); break;
        case 6: _t->skyboxModeChanged(); break;
        case 7: _t->setRadianceMapSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->setFragmentShader((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 9: _t->setFragmentShaderCode((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->setEnableIBL((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->setIblSampleCount((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->setIblRenderFrames((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->setSkyboxMode((*reinterpret_cast<std::add_pointer_t<enum SkyboxMode>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QQuick3DSkyMaterial::*)()>(_a, &QQuick3DSkyMaterial::radianceMapSizeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DSkyMaterial::*)()>(_a, &QQuick3DSkyMaterial::fragmentShaderChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DSkyMaterial::*)()>(_a, &QQuick3DSkyMaterial::fragmentShaderCodeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DSkyMaterial::*)()>(_a, &QQuick3DSkyMaterial::enableIBLChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DSkyMaterial::*)()>(_a, &QQuick3DSkyMaterial::iblSampleCountChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DSkyMaterial::*)()>(_a, &QQuick3DSkyMaterial::iblRenderFramesChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DSkyMaterial::*)()>(_a, &QQuick3DSkyMaterial::skyboxModeChanged, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->radianceMapSize(); break;
        case 1: *reinterpret_cast<QUrl*>(_v) = _t->fragmentShader(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->fragmentShaderCode(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->enableIBL(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->iblSampleCount(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->iblRenderFrames(); break;
        case 6: *reinterpret_cast<enum SkyboxMode*>(_v) = _t->skyboxMode(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRadianceMapSize(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setFragmentShader(*reinterpret_cast<QUrl*>(_v)); break;
        case 2: _t->setFragmentShaderCode(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setEnableIBL(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setIblSampleCount(*reinterpret_cast<int*>(_v)); break;
        case 5: _t->setIblRenderFrames(*reinterpret_cast<int*>(_v)); break;
        case 6: _t->setSkyboxMode(*reinterpret_cast<enum SkyboxMode*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QQuick3DSkyMaterial::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuick3DSkyMaterial::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QQuick3DSkyMaterialE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQuick3DPropertyChangedTracker"))
        return static_cast< QQuick3DPropertyChangedTracker*>(this);
    return QQuick3DObject::qt_metacast(_clname);
}

int QQuick3DSkyMaterial::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuick3DObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QQuick3DSkyMaterial::radianceMapSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuick3DSkyMaterial::fragmentShaderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuick3DSkyMaterial::fragmentShaderCodeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuick3DSkyMaterial::enableIBLChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuick3DSkyMaterial::iblSampleCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuick3DSkyMaterial::iblRenderFramesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuick3DSkyMaterial::skyboxModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
