/****************************************************************************
** Meta object code from reading C++ file 'qquick3dparticletargetdirection_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtquick3d/src/quick3dparticles/qquick3dparticletargetdirection_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquick3dparticletargetdirection_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN31QQuick3DParticleTargetDirectionE_t {};
} // unnamed namespace

template <> constexpr inline auto QQuick3DParticleTargetDirection::qt_create_metaobjectdata<qt_meta_tag_ZN31QQuick3DParticleTargetDirectionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QQuick3DParticleTargetDirection",
        "QML.Element",
        "TargetDirection3D",
        "QML.AddedInVersion",
        "1538",
        "positionChanged",
        "",
        "positionVariationChanged",
        "normalizedChanged",
        "magnitudeChanged",
        "magnitudeChangedVariation",
        "setPositionVariation",
        "QVector3D",
        "positionVariation",
        "setNormalized",
        "normalized",
        "setMagnitude",
        "magnitude",
        "setMagnitudeVariation",
        "magnitudeVariation",
        "position"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'positionChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'positionVariationChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'normalizedChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'magnitudeChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'magnitudeChangedVariation'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setPositionVariation'
        QtMocHelpers::SlotData<void(const QVector3D &)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'setNormalized'
        QtMocHelpers::SlotData<void(bool)>(14, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'setMagnitude'
        QtMocHelpers::SlotData<void(float)>(16, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 17 },
        }}),
        // Slot 'setMagnitudeVariation'
        QtMocHelpers::SlotData<void(float)>(18, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'position'
        QtMocHelpers::PropertyData<QVector3D>(20, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'positionVariation'
        QtMocHelpers::PropertyData<QVector3D>(13, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'normalized'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'magnitude'
        QtMocHelpers::PropertyData<float>(17, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'magnitudeVariation'
        QtMocHelpers::PropertyData<float>(19, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QQuick3DParticleTargetDirection, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QQuick3DParticleTargetDirection::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuick3DParticleDirection::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QQuick3DParticleTargetDirectionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QQuick3DParticleTargetDirectionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31QQuick3DParticleTargetDirectionE_t>.metaTypes,
    nullptr
} };

void QQuick3DParticleTargetDirection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QQuick3DParticleTargetDirection *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->positionChanged(); break;
        case 1: _t->positionVariationChanged(); break;
        case 2: _t->normalizedChanged(); break;
        case 3: _t->magnitudeChanged(); break;
        case 4: _t->magnitudeChangedVariation(); break;
        case 5: _t->setPositionVariation((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 6: _t->setNormalized((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->setMagnitude((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 8: _t->setMagnitudeVariation((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QQuick3DParticleTargetDirection::*)()>(_a, &QQuick3DParticleTargetDirection::positionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DParticleTargetDirection::*)()>(_a, &QQuick3DParticleTargetDirection::positionVariationChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DParticleTargetDirection::*)()>(_a, &QQuick3DParticleTargetDirection::normalizedChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DParticleTargetDirection::*)()>(_a, &QQuick3DParticleTargetDirection::magnitudeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DParticleTargetDirection::*)()>(_a, &QQuick3DParticleTargetDirection::magnitudeChangedVariation, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVector3D*>(_v) = _t->position(); break;
        case 1: *reinterpret_cast<QVector3D*>(_v) = _t->positionVariation(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->normalized(); break;
        case 3: *reinterpret_cast<float*>(_v) = _t->magnitude(); break;
        case 4: *reinterpret_cast<float*>(_v) = _t->magnitudeVariation(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPosition(*reinterpret_cast<QVector3D*>(_v)); break;
        case 1: _t->setPositionVariation(*reinterpret_cast<QVector3D*>(_v)); break;
        case 2: _t->setNormalized(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setMagnitude(*reinterpret_cast<float*>(_v)); break;
        case 4: _t->setMagnitudeVariation(*reinterpret_cast<float*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QQuick3DParticleTargetDirection::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuick3DParticleTargetDirection::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QQuick3DParticleTargetDirectionE_t>.strings))
        return static_cast<void*>(this);
    return QQuick3DParticleDirection::qt_metacast(_clname);
}

int QQuick3DParticleTargetDirection::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuick3DParticleDirection::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void QQuick3DParticleTargetDirection::positionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuick3DParticleTargetDirection::positionVariationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuick3DParticleTargetDirection::normalizedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuick3DParticleTargetDirection::magnitudeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuick3DParticleTargetDirection::magnitudeChangedVariation()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
