/****************************************************************************
** Meta object code from reading C++ file 'qquick3dmorphtarget_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtquick3d/src/quick3d/qquick3dmorphtarget_p.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquick3dmorphtarget_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19QQuick3DMorphTargetE_t {};
} // unnamed namespace

template <> constexpr inline auto QQuick3DMorphTarget::qt_create_metaobjectdata<qt_meta_tag_ZN19QQuick3DMorphTargetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QQuick3DMorphTarget",
        "QML.Element",
        "MorphTarget",
        "QML.AddedInVersion",
        "1536",
        "weightChanged",
        "",
        "attributesChanged",
        "setWeight",
        "castsShadows",
        "setAttributes",
        "QQuick3DMorphTarget::MorphTargetAttributes",
        "attributes",
        "weight",
        "MorphTargetAttributes",
        "MorphTargetAttribute",
        "Position",
        "Normal",
        "Tangent",
        "Binormal",
        "TexCoord0",
        "TexCoord1",
        "Color"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'weightChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'attributesChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setWeight'
        QtMocHelpers::SlotData<void(float)>(8, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 9 },
        }}),
        // Slot 'setAttributes'
        QtMocHelpers::SlotData<void(QQuick3DMorphTarget::MorphTargetAttributes)>(10, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'weight'
        QtMocHelpers::PropertyData<float>(13, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'attributes'
        QtMocHelpers::PropertyData<MorphTargetAttributes>(12, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'MorphTargetAttribute'
        QtMocHelpers::EnumData<enum MorphTargetAttribute>(15, 15, QMC::EnumIsScoped).add({
            {   16, MorphTargetAttribute::Position },
            {   17, MorphTargetAttribute::Normal },
            {   18, MorphTargetAttribute::Tangent },
            {   19, MorphTargetAttribute::Binormal },
            {   20, MorphTargetAttribute::TexCoord0 },
            {   21, MorphTargetAttribute::TexCoord1 },
            {   22, MorphTargetAttribute::Color },
        }),
        // flag 'MorphTargetAttributes'
        QtMocHelpers::EnumData<MorphTargetAttributes>(14, 15, QMC::EnumIsFlag | QMC::EnumIsScoped).add({
            {   16, MorphTargetAttribute::Position },
            {   17, MorphTargetAttribute::Normal },
            {   18, MorphTargetAttribute::Tangent },
            {   19, MorphTargetAttribute::Binormal },
            {   20, MorphTargetAttribute::TexCoord0 },
            {   21, MorphTargetAttribute::TexCoord1 },
            {   22, MorphTargetAttribute::Color },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QQuick3DMorphTarget, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QQuick3DMorphTarget::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuick3DObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QQuick3DMorphTargetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QQuick3DMorphTargetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19QQuick3DMorphTargetE_t>.metaTypes,
    nullptr
} };

void QQuick3DMorphTarget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QQuick3DMorphTarget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->weightChanged(); break;
        case 1: _t->attributesChanged(); break;
        case 2: _t->setWeight((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 3: _t->setAttributes((*reinterpret_cast<std::add_pointer_t<QQuick3DMorphTarget::MorphTargetAttributes>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QQuick3DMorphTarget::*)()>(_a, &QQuick3DMorphTarget::weightChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DMorphTarget::*)()>(_a, &QQuick3DMorphTarget::attributesChanged, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<float*>(_v) = _t->weight(); break;
        case 1: QtMocHelpers::assignFlags<MorphTargetAttributes>(_v, _t->attributes()); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setWeight(*reinterpret_cast<float*>(_v)); break;
        case 1: _t->setAttributes(*reinterpret_cast<MorphTargetAttributes*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QQuick3DMorphTarget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuick3DMorphTarget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QQuick3DMorphTargetE_t>.strings))
        return static_cast<void*>(this);
    return QQuick3DObject::qt_metacast(_clname);
}

int QQuick3DMorphTarget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuick3DObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void QQuick3DMorphTarget::weightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuick3DMorphTarget::attributesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
