/****************************************************************************
** Meta object code from reading C++ file 'qquick3dxranchormanager_openxr_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qtquick3d/src/xr/quick3dxr/openxr/qquick3dxranchormanager_openxr_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquick3dxranchormanager_openxr_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN23QQuick3DXrAnchorManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto QQuick3DXrAnchorManager::qt_create_metaobjectdata<qt_meta_tag_ZN23QQuick3DXrAnchorManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QQuick3DXrAnchorManager",
        "anchorAdded",
        "",
        "QQuick3DXrSpatialAnchor*",
        "anchor",
        "anchorRemoved",
        "QUuid",
        "uuid",
        "anchorUpdated",
        "sceneCaptureCompleted"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'anchorAdded'
        QtMocHelpers::SignalData<void(QQuick3DXrSpatialAnchor *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'anchorRemoved'
        QtMocHelpers::SignalData<void(QUuid)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'anchorUpdated'
        QtMocHelpers::SignalData<void(QQuick3DXrSpatialAnchor *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'sceneCaptureCompleted'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QQuick3DXrAnchorManager, qt_meta_tag_ZN23QQuick3DXrAnchorManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QQuick3DXrAnchorManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QQuick3DXrAnchorManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QQuick3DXrAnchorManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23QQuick3DXrAnchorManagerE_t>.metaTypes,
    nullptr
} };

void QQuick3DXrAnchorManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QQuick3DXrAnchorManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->anchorAdded((*reinterpret_cast<std::add_pointer_t<QQuick3DXrSpatialAnchor*>>(_a[1]))); break;
        case 1: _t->anchorRemoved((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1]))); break;
        case 2: _t->anchorUpdated((*reinterpret_cast<std::add_pointer_t<QQuick3DXrSpatialAnchor*>>(_a[1]))); break;
        case 3: _t->sceneCaptureCompleted(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QQuick3DXrAnchorManager::*)(QQuick3DXrSpatialAnchor * )>(_a, &QQuick3DXrAnchorManager::anchorAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DXrAnchorManager::*)(QUuid )>(_a, &QQuick3DXrAnchorManager::anchorRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DXrAnchorManager::*)(QQuick3DXrSpatialAnchor * )>(_a, &QQuick3DXrAnchorManager::anchorUpdated, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DXrAnchorManager::*)()>(_a, &QQuick3DXrAnchorManager::sceneCaptureCompleted, 3))
            return;
    }
}

const QMetaObject *QQuick3DXrAnchorManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuick3DXrAnchorManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QQuick3DXrAnchorManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuick3DXrAnchorManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QQuick3DXrAnchorManager::anchorAdded(QQuick3DXrSpatialAnchor * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QQuick3DXrAnchorManager::anchorRemoved(QUuid _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QQuick3DXrAnchorManager::anchorUpdated(QQuick3DXrSpatialAnchor * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QQuick3DXrAnchorManager::sceneCaptureCompleted()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
