/****************************************************************************
** Meta object code from reading C++ file 'qquick3dquadtextureprovider_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtquick3d/src/helpers/qquick3dquadtextureprovider_p.h"
#include <QtNetwork/QSslError>
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquick3dquadtextureprovider_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN27QQuick3DQuadTextureProviderE_t {};
} // unnamed namespace

template <> constexpr inline auto QQuick3DQuadTextureProvider::qt_create_metaobjectdata<qt_meta_tag_ZN27QQuick3DQuadTextureProviderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QQuick3DQuadTextureProvider",
        "QML.Element",
        "QuadTextureProvider",
        "QML.AddedInVersion",
        "1548",
        "fragmentShaderChanged",
        "",
        "fragmentShaderCodeChanged",
        "widthChanged",
        "heightChanged",
        "formatChanged",
        "fragmentShader",
        "QUrl",
        "fragmentShaderCode",
        "width",
        "height",
        "format",
        "QQuick3DTextureData::Format"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fragmentShaderChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fragmentShaderCodeChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'widthChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'heightChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'formatChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'fragmentShader'
        QtMocHelpers::PropertyData<QUrl>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0, 0x60c),
        // property 'fragmentShaderCode'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1, 0x60c),
        // property 'width'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2, 0x60c),
        // property 'height'
        QtMocHelpers::PropertyData<int>(15, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 3, 0x60c),
        // property 'format'
        QtMocHelpers::PropertyData<QQuick3DTextureData::Format>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 4, 0x60c),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QQuick3DQuadTextureProvider, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN27QQuick3DQuadTextureProviderE[] = {
    QMetaObject::SuperData::link<QQuick3DTextureData::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject QQuick3DQuadTextureProvider::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuick3DTextureProviderExtension::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QQuick3DQuadTextureProviderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QQuick3DQuadTextureProviderE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN27QQuick3DQuadTextureProviderE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27QQuick3DQuadTextureProviderE_t>.metaTypes,
    nullptr
} };

void QQuick3DQuadTextureProvider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QQuick3DQuadTextureProvider *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fragmentShaderChanged(); break;
        case 1: _t->fragmentShaderCodeChanged(); break;
        case 2: _t->widthChanged(); break;
        case 3: _t->heightChanged(); break;
        case 4: _t->formatChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QQuick3DQuadTextureProvider::*)()>(_a, &QQuick3DQuadTextureProvider::fragmentShaderChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DQuadTextureProvider::*)()>(_a, &QQuick3DQuadTextureProvider::fragmentShaderCodeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DQuadTextureProvider::*)()>(_a, &QQuick3DQuadTextureProvider::widthChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DQuadTextureProvider::*)()>(_a, &QQuick3DQuadTextureProvider::heightChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QQuick3DQuadTextureProvider::*)()>(_a, &QQuick3DQuadTextureProvider::formatChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->fragmentShader(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->fragmentShaderCode(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->width(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->height(); break;
        case 4: *reinterpret_cast<QQuick3DTextureData::Format*>(_v) = _t->format(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFragmentShader(*reinterpret_cast<QUrl*>(_v)); break;
        case 1: _t->setFragmentShaderCode(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setWidth(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setHeight(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setFormat(*reinterpret_cast<QQuick3DTextureData::Format*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QQuick3DQuadTextureProvider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuick3DQuadTextureProvider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QQuick3DQuadTextureProviderE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQuick3DPropertyChangedTracker"))
        return static_cast< QQuick3DPropertyChangedTracker*>(this);
    return QQuick3DTextureProviderExtension::qt_metacast(_clname);
}

int QQuick3DQuadTextureProvider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuick3DTextureProviderExtension::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void QQuick3DQuadTextureProvider::fragmentShaderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuick3DQuadTextureProvider::fragmentShaderCodeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuick3DQuadTextureProvider::widthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuick3DQuadTextureProvider::heightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuick3DQuadTextureProvider::formatChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
